;; Copyright 2009, 2010, 2011, 2013 Kevin Ryde.

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(require 'cl)

;; for display-warnings "type"
(defgroup my-load-all nil "My Load All."
  :prefix "my-load-all-")

;; gnus not ask about registry
(setq gnus-registry-install nil)

;; viper.el not ask about viperizing
(setq viper-mode nil)

;; idlwave.el no buffer scans
(setq idlwave-scan-all-buffers-for-routine-info nil)

(defun my-load-all--custom ()
  "Load all libraries mentioned in `custom-loads' of defcustom variables."
  (lwarn 'my-load-all 'debug
         "Loading all custom-load autoloads ...")
  (require 'cus-start)
  (require 'cus-load)

  (mapatoms
   (lambda (symbol)
     (dolist (name (get symbol 'custom-loads))
       (my-load-all--one-library name
                                 (lambda ()
                                   (format "Symbol `%s'" symbol))))))

  (lwarn 'my-load-all 'debug
         "done"))

(defun my-load-all--custom-groups ()
  "Load all libraries mentioned in `custom-loads' of defgroup forms."
  (lwarn 'my-load-all 'debug
         "Loading all custom-group custom-loads ...")
  (require 'cus-start)
  (require 'cus-load)

  (mapatoms
   (lambda (symbol)
     (if (get symbol 'custom-group)
         (dolist (name (get symbol 'custom-loads))
           (my-load-all--one-library name
                                     (lambda ()
                                       (format "Symbol `%s'" symbol)))))))

  (lwarn 'my-load-all 'debug
         "done"))

(defun my-load-all--autoloads ()
  "Load all autoloaded functions."

  (lwarn 'my-load-all 'debug
         "Loading all autoloads ...")

  ;; FIXME: a library might create autoloads for its sub-parts ...
  (mapatoms
   (lambda (symbol)
     (when (fboundp symbol)
       (let ((func (symbol-function symbol)))
         (when (eq 'autoload (car-safe func))
           (my-load-all--one-library (cadr func)
                                     (lambda ()
                                       (format "autoload of `%s'" symbol))))))))

  (lwarn 'my-load-all 'debug
         "done"))

;; (setq foo (my-load-all--libraries-list))
;; (setq my-path (my-load-all--pruned-path))

(defun my-load-all--pruned-path ()
  "Return `load-path' with some directories pruned out."
  (let ((path load-path))
    (setq path (remove "." path))
    (setq path (remove (expand-file-name ".") path))
    (setq path (remove "~/lisp" path))
    (setq path (remove (expand-file-name "~/lisp") path))
    path))

(defun my-load-all--libraries-list ()
  "Return a list of all available lisp libraries in `load-path'.
Duplicates are suppressed, and certain non-library packages are
excluded, such as dgnushack."

  (let ((lst (mapcan (lambda (path)
                       (condition-case err
                           (directory-files path t "\\.elc?\\'")
                         (error nil)))
                     (my-load-all--pruned-path))))

    (setq lst (mapcar (lambda (filename)
                        (file-name-sans-extension
                         (file-name-nondirectory filename)))
                      lst))

    (setq lst (remove* "\\`\\.#" lst :test 'string-match))
    (setq lst (remove* "~\\'"    lst :test 'string-match))

    
    (setq lst (remove "tests" lst)) ;; my pchart tests.el
    (setq lst (remove "dgnushack" lst))
    (setq lst (remove "dos-w32" lst))    ;; ms-dos specific
    (setq lst (remove "w32-fns" lst))    ;; ms-dos specific
    (setq lst (remove "edt-mapper" lst)) ;; script to create mapping
    (setq lst (remove "remember-experimental" lst)) ;; does a compose-mail
    (setq lst (remove "planner-experimental" lst))  ;; maybe similar

    (unless (featurep 'xemacs)
      (setq lst (remove* "xmas\\'" lst :test 'string-match)))

    (unless (equal emacs-major-version 20)
      (setq lst (remove* "-20\\'" lst :test 'string-match))) ;; apel

    (unless (equal emacs-major-version 21)
      (setq lst (remove* "-e21\\'" lst :test 'string-match))
      (setq lst (remove* "-emacs21\\'" lst :test 'string-match)))

    (let ((seen (make-hash-table :test 'equal)))
      (remove-if (lambda (name)
                   (if (gethash name seen)
                       t
                     (puthash name t seen)
                     nil))
                 lst))))

(defun my-load-all--libraries ()
  "Load all libraries available in the `load-path'."

  (lwarn 'my-load-all 'debug
         "Loading all libraries ...")

  (dolist (name (my-load-all--libraries-list))
    (my-load-all--one-library name))

  (lwarn 'my-load-all 'debug
         "done"))


;; (flet ((kill-emacs (&rest args) nil))

(defun my-load-all--one-library (name &optional origin)
  (if (symbolp name)
      (setq name (symbol-name name)))
  (cond ((string-match "icicles" name)
         (message "skip icicles for now"))

        ((symbolp name)
         (unless (featurep name)
           (lwarn 'my-load-all :debug "load %S" name)
           (condition-case err
               (require name)
             (error
              (if (functionp origin)
                  (setq origin (funcall origin)))
              (lwarn 'my-load-all 'error
                     "%scannot require %S: %s"
                     (my-load-all--origin-colon origin)
                     name err)))))

        ;; skip if previously loaded
        ((assoc name load-history))
        ((assoc (locate-library name) load-history))

        (t
         (lwarn 'my-load-all 'debug "load %S" name)
         (condition-case err
             (load name)
           (error
            (lwarn 'my-load-all 'error
                   "%scannot load %S: %s"
                   (my-load-all--origin-colon origin)
                   name err)))))

  (if (member "../wip" load-path)
      (error "oops %S" (my-load-all--origin-colon origin))))

(defun my-load-all--origin-colon (origin)
  (if (functionp origin)
      (setq origin (funcall origin)))
  (if origin
      (concat origin ": ")
    ""))

(defun my-load-all--one-function (symbol)
  (let ((func (symbol-function symbol)))
    (if (autoloadp func)
        (my-load-all--one-library (cadr func)
                                  (lambda ()
                                    (format "autoload of `%s'" symbol))))))

(provide 'my-load-all)
