;; autoloads-mangle.el -- mangle generated autoloads a bit

;; Copyright 2009, 2010, 2015 Kevin Ryde.

;; Author: Kevin Ryde <user42_kevin@yahoo.com.au>
;; Keywords: maint

;; This file is part of miscbits-el.
;;
;; miscbits-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; miscbits-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with miscbits-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

;;; Commentary:

;;; Code:

;;-----------------------------------------------------------------------------
;; generic

(defun autoloads-mangle-remove-trailing-nils (lst)
  "Return list LST with any trailing nil's removed.
This is non-destructive, and the return is just LST if there's no nils to remove."
  (while (and (consp lst)
              (eq nil (car (last lst))))
    (setq lst (butlast lst 1)))
  lst)
  
(defun autoloads-mangle-ad-advice-enabled-p (func class name)
  "Return non-nil if FUNC advice CLASS and NAME is enabled."
  (let ((advice (ad-find-advice func class name)))
    (and advice
         (ad-advice-enabled advice))))

(defmacro autoloads-mangle-with-ad-enable-advice (func class name &rest body)
  "Evaluate BODY with FUNC advice CLASS and NAME temporarily enabled.
And `unwind-protect' ensures the advice is re-disabled (if it was
disabled to start with) no matter what BODY does.

In the current implementation CLASS must be a specific class,
like `before' or `around', it can't be `any'."

  `(let ((autoloads-mangle-with-ad-enable-advice--old
          (autoloads-mangle-ad-advice-enabled-p ,func ,class ,name)))
     (unwind-protect
         (progn
           (unless autoloads-mangle-with-ad-enable-advice--old
             (ad-enable-advice ,func ,class ,name)
             (ad-activate ,func))
           ,@body)
       (unless autoloads-mangle-with-ad-enable-advice--old
         (ad-disable-advice ,func ,class ,name)
         (ad-activate ,func)))))
(put 'autoloads-mangle-with-ad-enable-advice 'lisp-indent-function 3)

(defun autoloads-mangle-string-first-line (str)
  "Return the first line of STR.
This means everything up to but not including the first newline,
or the whole string if no newlines at all."
  (if (string-match "\n" str)
      (substring str 0 (match-beginning 0))
    str))
;; (autoloads-mangle-string-first-line "abc\ndef\nghi")
;; (autoloads-mangle-string-first-line "abc")

;;-----------------------------------------------------------------------------

(defun autoloads-mangle-oneline-autoload-docstring (form)
  "If FORM is an (autoload ...) return it with a one line docstring.
Otherwise return FORM unchanged.

The idea here is to keep lots of verbiage out of autoloads,
leaving it for the full definition once loaded.  For
byte-compiled autoloads there's no need for this, as the dynamic
docstrings defer them until needed, but for plain .el a shorter
docstring may save a few bytes of core."

  ;;  0        1     2      3           4                5
  ;; (autoload 'func "file" "docstring" interactive-flag type)
  ;; (defvar   var   value  "docstring")
  ;;
  (if (eq 'progn (car-safe form))
      (mapcar 'autoloads-mangle-oneline-autoload-docstring form)

    (when (and (memq (car-safe form) '(autoload defvar))
               (stringp (nth 3 form)))
      (setq form (copy-sequence form))
      (setf (nth 3 form) (autoloads-mangle-string-first-line (nth 3 form))))
    form))

;; (autoloads-mangle-oneline-autoload-docstring nil)
;; (autoloads-mangle-oneline-autoload-docstring '(autoload 'func "file"))
;; (autoloads-mangle-oneline-autoload-docstring '(autoload 'func "file" "one\ntwo" interactive-flag))

(defadvice make-autoload (after autoloads-mangle-oneline-docstring disable)
  "Truncate to one line docstrings using `autoloads-mangle-oneline-autoload-docstring'."
  (setq ad-return-value
        (autoloads-mangle-oneline-autoload-docstring ad-return-value)))

(defun autoloads-mangle-generate-file-autoloads (filename)
  "Like `generate-file-autoloads' but with one-line docstrings."
  (let ((print-quoted t))
    (autoloads-mangle-with-ad-enable-advice 'make-autoload 'after 'autoloads-mangle-oneline-docstring
      (generate-file-autoloads filename))))


(provide 'autoloads-mangle)

;;; autoloads-mangle.el ends here
